﻿using TextFormatConverter.models;
using TextFormatConverter.converters;

namespace TextFormatConverter
{
    internal class Program
    {
        const string jsonAnalysis = @"..\..\..\..\analysis_in.json";
        const string jsonPeople = @"..\..\..\..\people_in.json";

        const string xmlAnalysis = @"..\..\..\..\analysis_in.xml";
        const string xmlPeople = @"..\..\..\..\people_in.xml";

        const string yamlAnalysis = @"..\..\..\..\analysis_in.yaml";
        const string yamlPeople = @"..\..\..\..\people_in.yaml";

        const string csvAnalysis = @"..\..\..\..\analysis_in.csv";
        const string csvPeople = @"..\..\..\..\people_in.csv";

        public static void PrintList<T>(List<T> list)
        {
            foreach (T item in list)
                Console.WriteLine(item);
        }

        static void switchTextReader(string fileName)
        {
            List<Analysis> analysis = new List<Analysis>();
            List<Person> people = new List<Person>();
            switch (fileName)
            {
                case jsonAnalysis:
                    analysis = Converter.ReadJson<Analysis>(fileName);
                    PrintList(analysis);
                    return;
                case jsonPeople:
                    people = Converter.ReadJson<Person>(fileName);
                    PrintList(people);
                    return;
                case xmlAnalysis:
                    analysis = Converter.ReadXml<Analysis>(fileName);
                    PrintList(analysis);
                    return;
                case xmlPeople:
                    people = Converter.ReadXml<Person>(fileName);
                    PrintList(people);
                    return;
                case yamlAnalysis:
                    analysis = Converter.ReadYaml<Analysis>(fileName);
                    PrintList(analysis);
                    return;
                case yamlPeople:
                    people = Converter.ReadYaml<Person>(fileName);
                    PrintList(people);
                    return;
                case csvAnalysis:
                    analysis = Converter.ReadCsv<Analysis>(fileName);
                    PrintList(analysis);
                    return;
                case csvPeople:
                    people = Converter.ReadCsv<Person>(fileName);
                    PrintList(people);
                    return;
                default:
                    Console.WriteLine("Такого файла нет");
                    return;
            }
        }

        static void switchTextWriter<T>(string desirserializationSource, string serializationPath, string format)
        {
            List<T> objects = Converter.ReadJson<T>(desirserializationSource);
            PrintList(objects);
            switch (format)
            {
                case "json":
                    Converter.WriteJson(objects, serializationPath);
                    return;
                case "xml":
                    Converter.WriteXml(objects, serializationPath);
                    return;
                case "yaml":
                    Converter.WriteYaml(objects, serializationPath);
                    return;
                case "csv":
                    Converter.WriteCsv(objects, serializationPath);
                    return;
                default:
                    Console.WriteLine("Такого формата нет");
                    return;
            }

        }

        static void UserMode()
        {
            do
            {
                List<Analysis> analysis = new List<Analysis>();
                List<Person> people = new List<Person>();
                Console.WriteLine("1 - Чтение\n2 - Запись\n0 - Выход\n");
                int query = Convert.ToInt32(Console.ReadLine());
                switch (query)
                {
                    case 0:
                        Console.WriteLine("Выход");
                        return;
                    case 1:
                        Console.WriteLine("Выберите файл для чтения: ");
                        Console.WriteLine(jsonAnalysis);
                        Console.WriteLine(jsonPeople);
                        Console.WriteLine(xmlAnalysis);
                        Console.WriteLine(xmlPeople);
                        Console.WriteLine(yamlAnalysis);
                        Console.WriteLine(yamlPeople);
                        Console.WriteLine(csvAnalysis);
                        Console.WriteLine(csvPeople);

                        string fileName = Console.ReadLine();
                        try
                        {
                            Console.WriteLine("Содержимое файла:");
                            string content = File.ReadAllText(fileName);
                            Console.Write(content);
                            Console.WriteLine("\n\nДесериализованный файл:");
                            switchTextReader(fileName);
                        }
                        catch
                        {
                            Console.WriteLine("Что-то пошло не так, повторите попытку");
                        }
                        break;
                    case 2:
                        Console.WriteLine("Хотите записать спислк анализов(0) или список людей(1)?");
                        int writeInd = Convert.ToInt32(Console.ReadLine());
                        string newFilePath = "";
                        string format = "";
                        switch (writeInd)
                        {
                            case 0:                                
                                Console.WriteLine("Введите имя/путь для нового файла:");
                                newFilePath = Console.ReadLine();
                                Console.WriteLine("Введите формат сериализации (xml, yaml, csv, json):");
                                format = Console.ReadLine();
                                Console.WriteLine("Вы выбрали лист анализов, он выглядит так:");

                                switchTextWriter<Analysis>(jsonAnalysis, newFilePath, format);
                                break;
                            case 1:
                                Console.WriteLine("Введите имя/путь для нового файла:");
                                newFilePath = Console.ReadLine();
                                Console.WriteLine("Введите формат сериализации (xml, yaml, csv, json):");
                                format = Console.ReadLine();
                                Console.WriteLine("Вы выбрали лист людей, он выглядит так:");

                                switchTextWriter<Person>(jsonPeople, newFilePath, format);
                                break;
                            default:
                                Console.WriteLine("Такого варианта нет");
                                break;
                        }
                        Console.WriteLine("В файл записано: ");
                        Console.WriteLine(File.ReadAllText(newFilePath));
                        break;
                    default:
                        Console.WriteLine("Такой команды нет, повторите ввод\n");
                        break;
                }

            } while (true);
        }

        static void Main(string[] args)
        {
            UserMode();
        }
    }
}