﻿using Newtonsoft.Json.Linq;
using System.Text;
using Newtonsoft.Json;
using YamlDotNet.Serialization;
using YamlDotNet.Serialization.NamingConventions;
using System.Xml.Serialization;
using CsvHelper.Configuration;
using CsvHelper;
using System.Globalization;

namespace TextFormatConverter.converters
{
    internal static class Converter
    {
        public static List<T>? ReadJson<T>(string path)
        {
            try
            {
                string jsonString = File.ReadAllText(path);
                JArray jArray = JArray.Parse(jsonString);
                List<T>? objects = jArray.ToObject<List<T>>();
                return objects;
            }
            catch (Exception ex)
            {
                Console.WriteLine($"Исключение: {ex.Message}\n" +
                    $"Метод: {ex.TargetSite}\n" +
                    $"Трассировка стека: {ex.StackTrace}");
                return null;
            }
        }
        public static bool WriteJson<T>(List<T> objects, string path)
        {
            try
            {
                string? newJson = JsonConvert.SerializeObject(objects, Newtonsoft.Json.Formatting.Indented);
                File.WriteAllText(path, newJson);
                return true;
            }
            catch (Exception ex)
            {
                Console.WriteLine($"Исключение: {ex.Message}\n" +
                    $"Метод: {ex.TargetSite}\n" +
                    $"Трассировка стека: {ex.StackTrace}");
                return false;
            }
        }
        public static List<T>? ReadYaml<T>(string path)
        {
            try
            {
                string yamlString = File.ReadAllText(path);
                IDeserializer deserializer = new DeserializerBuilder()
                    .Build();
                List<T> objects = deserializer.Deserialize<List<T>>(yamlString);
                return objects;
            }
            catch (Exception ex)
            {
                Console.WriteLine($"Исключение: {ex.Message}\n" +
                    $"Метод: {ex.TargetSite}\n" +
                    $"Трассировка стека: {ex.StackTrace}");
                return null;
            }

        }        
        public static bool WriteYaml<T>(List<T> objects, string path)
        {
            try
            {
                ISerializer serializer = new SerializerBuilder()
                    .WithNamingConvention(CamelCaseNamingConvention.Instance)
                    .Build();
                string newYaml = serializer.Serialize(objects);
                File.WriteAllText(path, newYaml);
                return true;
            }
            catch (Exception ex)
            {
                Console.WriteLine($"Исключение: {ex.Message}\n" +
                    $"Метод: {ex.TargetSite}\n" +
                    $"Трассировка стека: {ex.StackTrace}");
                return false;
            }
        }       
        public static List<T>? ReadXml<T>(string path)
        {
            try
            {
                XmlSerializer formatter = new XmlSerializer(typeof(List<T>));
                using (FileStream fs = new FileStream(path, FileMode.OpenOrCreate))
                {
                    return formatter.Deserialize(fs) as List<T>;
                }
            }
            catch (Exception ex)
            {
                Console.WriteLine($"Исключение: {ex.Message}\n" +
                    $"Метод: {ex.TargetSite}\n" +
                    $"Трассировка стека: {ex.StackTrace}");
                return null;
            }
        }        
        public static bool WriteXml<T>(List<T> objects, string path)
        {
            try
            {
                XmlSerializer formatter = new XmlSerializer(typeof(List<T>));
                using (FileStream fs = new FileStream(path, FileMode.OpenOrCreate))
                {
                    formatter.Serialize(fs, objects);
                }
                return true;
            }
            catch (Exception ex)
            {
                Console.WriteLine($"Исключение: {ex.Message}\n" +
                    $"Метод: {ex.TargetSite}\n" +
                    $"Трассировка стека: {ex.StackTrace}");
                return false;
            }
        }

        public static List<T>? ReadCsv<T>(string path)
        {
            var config = new CsvConfiguration(CultureInfo.CurrentCulture)
            {
                HasHeaderRecord = true,
                Delimiter = ";",
                Encoding = Encoding.UTF8,
                MemberTypes = CsvHelper.Configuration.MemberTypes.Fields
            };
            try
            {
                using (var reader = new StreamReader(path))
                using (var csv = new CsvReader(reader, config))
                {
                    return csv.GetRecords<T>().ToList();
                }
            }
            catch (Exception ex)
            {
                Console.WriteLine($"Исключение: {ex.Message}\n" +
                    $"Метод: {ex.TargetSite}\n" +
                    $"Трассировка стека: {ex.StackTrace}");
                return null;
            }
        }

        public static bool WriteCsv<T>(List<T> objects, string path)
        {
            try
            {
                var config = new CsvConfiguration(CultureInfo.CurrentCulture)
                {
                    HasHeaderRecord = true,
                    Delimiter = ";",
                    Encoding = Encoding.UTF8,
                    MemberTypes = CsvHelper.Configuration.MemberTypes.Fields
                };

                using (var writer = new StreamWriter(path))
                using (var csv = new CsvWriter(writer, config))
                {
                    csv.WriteHeader<T>();
                    csv.NextRecord();
                    foreach (var record in objects)
                    {
                        csv.WriteRecord(record);
                        csv.NextRecord();
                    }
                }
                return true;
            }
            catch (Exception ex)
            {
                Console.WriteLine($"Исключение: {ex.Message}\n" +
                    $"Метод: {ex.TargetSite}\n" +
                    $"Трассировка стека: {ex.StackTrace}");
                return false;
            }
        }
    }
}
